﻿//Example program for the application note
//1MA268 How to use Rohde & Schwarz IVI.NET instrument drivers
//http://www.rohde-schwarz.com/appnote/1MA268
//Author: Miloslav Macko

// Version 3.0.0
// - updated for RsScope driver 5.10.1

// Version 2.0.0
// - updated for RsScope driver 3.0.0, replaced all IVINET.Internal exceptions with Ivi.Driver exceptions
// - removed extended method for synchronization - no longer required, since the driver has now both *OPC? and *WAI method built-in
//
// Version 1.0.0
//- Initial Version

using System;
using System.Text.RegularExpressions;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Threading;
using System.Diagnostics;
using System.Globalization;

//----------------------------------------------------------------------------------------------------------------
#region 3.2 Adding assemblies references (64-bit target application)
//Add the Reference "c:\Program Files\IVI Foundation\IVI\Microsoft.NET\Framework64\v2.0.50727\IviFoundationSharedComponents 1.1.0\Ivi.Driver.dll"
using Ivi.Driver;

//Add the Reference "c:\Program Files\IVI Foundation\IVI\Microsoft.NET\Framework64\v2.0.50727\IviFoundationSharedComponents 1.1.0\Ivi.Scope.dll"
using Ivi.Scope;

//Add the Reference "c:\Program Files\Rohde-Schwarz\RsScope\Bin\RohdeSchwarz.RsScope.Fx40.dll"
using RohdeSchwarz.RsScope;

//Add extension functions (found at the end of this file)
using RsScopeExtensions;

#endregion
//----------------------------------------------------------------------------------------------------------------
namespace _1MA268_RsScopeIVI.NET
{
    class Program
    {
        static void Main(string[] args)
        {
            //set the regional settings to fixed: decimal point will always be '.'
            Thread.CurrentThread.CurrentCulture = CultureInfo.InvariantCulture;
            string exceptionMessage = null;
            RsScope driver = null;
            try
            {
                #region 3.3 Initializing of a HISLIP session
                driver = new RsScope("TCPIP::10.205.0.103::hislip0", true, true, "Simulate=False");
                
                //reset the device separately
                driver.Utility.Reset();

                //Display ON in remote
                driver.Display.DisplayUpdate = true;

                //switch the simulation mode. Default value: false
                driver.DriverOperation.Simulate = false;
                
                //the driver checks the range of the entered parameters. Default value: true
                driver.DriverOperation.RangeCheck = true;

                //query the device status "*STB?" after every command. Default value: true
                driver.DriverOperation.QueryInstrumentStatus = true;
                
                //retrieving the driver io-resource description and logical name
                string ioResource = driver.DriverOperation.IOResourceDescriptor;
                string logName = driver.DriverOperation.LogicalName;
                Console.WriteLine("Driver session IO-resource: {0}", ioResource);
                Console.WriteLine("Driver session logical name: {0}", logName);
                #endregion
                //----------------------------------------------------------------------------------------------------------------
                #region 3.4 Reading basic info
                //reading the channel count from the instrument
                int channelCount = driver.System.ChannelCount;
                string pcName = driver.System.ComputerName;
                string fwVersion = driver.System.FirmwareVersion;

                Console.WriteLine("Instrument PC name: {0}", pcName);
                Console.WriteLine("Instrument channels #: {0}", channelCount);
                Console.WriteLine("Instrument FW version: {0}", fwVersion);

                Console.Write("\nRepCap '{0}' count {1}, all values: ", driver.Channel.Name, driver.Channel.Count);
                foreach (var element in driver.Channel)
                    Console.Write("'{0}' ", element.Name);

                //in case an aquisition is running, stop it acquisition - set the single mode
                driver.WaveformAcquisition.Stop();
                
                //set display update in remote-control to ON
                driver.Display.DisplayUpdate = true;
                #endregion
                //----------------------------------------------------------------------------------------------------------------
                #region 3.5 Direct SCPI write/query
                //direct string write - use for the commands not for the queries
                driver.UtilityFunctions.WriteCommand("SYST:DISP:UPD ON"); //same effect as driver.Display.DisplayUpdate = true;
                
                //direct string query
                string idnResponse = null;
                int repetitions = 1000;
                var stopWatch = new Stopwatch();

                //perform 1000x *IDN? query and evaluate an average execution time
                stopWatch.Reset();
                stopWatch.Start();
                for (int i = 0; i < repetitions; i++)
                {
                    idnResponse = driver.UtilityFunctions.QueryString("*IDN?"); //same as driver.UtilityFunctions.IDQueryResponse;
                }
                stopWatch.Stop();
                Console.WriteLine($"\n\n*IDN? response: {idnResponse}. Average time over {repetitions} repetitions: {stopWatch.Elapsed.TotalSeconds / repetitions:F4} seconds.");
                #endregion
                //----------------------------------------------------------------------------------------------------------------
                #region 3.6 Channels setup, using repeated capabilities
                //horizontal setting - Acquisition time and Record Length
                driver.Acquisition.HorizontalRecordSelection = RecordSelection.RecordTime;
                driver.Acquisition.HorizontalTimePerRecord = Ivi.Driver.PrecisionTimeSpan.FromMilliseconds(10); //10ms
                driver.Acquisition.HorizontalRecordLength = 1002;

                //configuring the Channel 1
                var CH1 = driver.Channel["CH1"];
                CH1.Configure(2.000, 0.0, RohdeSchwarz.RsScope.VerticalCoupling.AC, true);
                
                //configuring the Channel 2
                var CH2 = driver.Channel["CH2"];
                CH2.Configure(5.000, 0.0, RohdeSchwarz.RsScope.VerticalCoupling.AC, true);
                #endregion
                //----------------------------------------------------------------------------------------------------------------
                #region 3.7 Trigger settings
                //set up the trigger
                var TriggerA = driver.Trigger["TrigA"];
                TriggerA.Modifier = RohdeSchwarz.RsScope.TriggerModifier.Auto;
                TriggerA.Timeout.TimeoutValue = 13.0;
                TriggerA.Source = RohdeSchwarz.RsScope.TriggerSource.Channel1;
                TriggerA.Type = RohdeSchwarz.RsScope.TriggerType.Edge;
                TriggerA.Edge.Slope = RohdeSchwarz.RsScope.Slope.Positive;
                TriggerA.Channel["TsCH1"].Level = 0.3; //setting the trigger level on CH_1 to 0.3V
                #endregion
                //----------------------------------------------------------------------------------------------------------------
                #region 3.8.2 Acquisition #1 and reading waveforms
                //Acquisition #1: arming the RTx
                Console.WriteLine("\n\n----- Acquisition #1:");

                driver.WaveformAcquisition.RunSingleWithoutWait();

                //here adjust your DUT to produce a signal - DUT.GenerateBurst()

                //wait until the acquisition has finished
                driver.WaveformAcquisition.WaitForMeasurementComplete(10000); //10000ms

                //copy the waveforms from Channel 1 and Channel 2 to the PC
                IWaveform<double> waveform_CH1 = null;
                IWaveform<double> waveform_CH2 = null;

                waveform_CH1 = CH1.Waveform["W0"].FetchWaveform(waveform_CH1);
                waveform_CH2 = CH2.Waveform["W0"].FetchWaveform(waveform_CH2);
                //Acquisition #1 parameters
                Console.WriteLine("Trigger source: {0}", TriggerA.Source);
                Console.WriteLine("CH1 Samples count: {0}, CH2 Samples count: {1}", waveform_CH1.Capacity, waveform_CH2.Capacity);
                #endregion
                //----------------------------------------------------------------------------------------------------------------
                #region 3.8.3 Acquisition #2 and converting waveforms
                //Acquisition #2: with increased number of samples (higher record length)
                int newRecordLength = 1100;
                driver.Acquisition.HorizontalRecordLength = newRecordLength;
                //start the new acquisition and wait for it to finish, timeout = 3s
                driver.UtilityFunctions.OPCTimeout = 3000;

                //to simulate the acquisition timeout, change the trigger source to Channel 2:
                TriggerA.Source = RohdeSchwarz.RsScope.TriggerSource.Channel2;
                TriggerA.Channel["TsCH2"].Level = 0.2; //setting the trigger level on CH_2 to 0.2V
                
                //custom method for driver.WaveformAcquisition.RunSingle() with repeat workaround
                Console.WriteLine("\n\n----- Acquisition #2:");
                bool success = driver.Extended_RunSigleWithRepeat(TriggerA, 5);

                //increase the waveforms capacity and fetch the data from RTx
                if (success)
                {
                    waveform_CH1.Capacity = newRecordLength;
                    waveform_CH2.Capacity = newRecordLength;
                    waveform_CH1 = CH1.Waveform["W0"].FetchWaveform(waveform_CH1);
                    waveform_CH2 = CH2.Waveform["W0"].FetchWaveform(waveform_CH2);
                    
                    //Acquisition #2 parameters
                    Console.WriteLine("Trigger source: {0}", TriggerA.Source);
                    Console.WriteLine("CH1 Samples count: {0}, CH2 Samples count: {1}", waveform_CH1.Capacity, waveform_CH2.Capacity);
                }
                else
                {
                    Console.WriteLine("Acquisition #2 is cancelled.\nBoth waveforms still contain the data from the Acquisition #1.");
                }
                
                //stop the acquisition in case the trigger event has not arrived
                driver.WaveformAcquisition.Stop();

                //convert from waveforms to arrays
                double[] samplesArray_CH1 = new double[waveform_CH1.Capacity];
                double[] samplesArray_CH2 = new double[waveform_CH2.Capacity];

                samplesArray_CH1 = waveform_CH1.GetAllElements();
                samplesArray_CH2 = waveform_CH2.GetAllElements();
                #endregion
                //----------------------------------------------------------------------------------------------------------------
                #region 3.9 Measurements
                var Meas1 = driver.Measurements["Meas1"];
                var Meas2 = driver.Measurements["Meas2"];
                var Meas3 = driver.Measurements["Meas3"];

                //measurement #1
                Meas1.AmplitudeTime.MainMeasurement = AmplitudeTimeMeasurementType.Amplitude;
                Meas1.GeneralSettings.Source(WaveformParameter.Channel1Waveform1, WaveformParameter.Channel1Waveform1);
                Meas1.GeneralSettings.Enabled = true;
                //measurement #2
                Meas2.AmplitudeTime.MainMeasurement = AmplitudeTimeMeasurementType.Frequency;
                Meas2.GeneralSettings.Source(WaveformParameter.Channel1Waveform1, WaveformParameter.Channel1Waveform1);
                Meas2.GeneralSettings.Enabled = true;
                //measurement #3
                Meas3.AmplitudeTime.MainMeasurement = AmplitudeTimeMeasurementType.Amplitude;
                Meas3.GeneralSettings.Source(WaveformParameter.Channel2Waveform1, WaveformParameter.Channel2Waveform1);
                Meas3.GeneralSettings.Enabled = true;

                //Continue only when all previous commands were processed
                var opc = driver.UtilityFunctions.QueryOPC();
                                
                Console.WriteLine("Current waveforms measurement results:");
                Console.WriteLine("CH1 Amplitude {0:F3} V", Meas1.Results.FetchMainMeasurement(StatisticsType.Actual));
                Console.WriteLine("CH1 Frequency {0:F3} Hz", Meas2.Results.FetchMainMeasurement(StatisticsType.Actual));
                Console.WriteLine("CH2 Amplitude {0:F3} V", Meas3.Results.FetchMainMeasurement(StatisticsType.Actual));
                #endregion
                //----------------------------------------------------------------------------------------------------------------
                #region 3.10 Exporting the Waveforms to a csv file
                string csvFileName = @"c:\Temp\data.csv";
                string columnSeparator = ";";
                string rowSeparator = "\n";
                double sampleTime;
                var csvFile = new StreamWriter(csvFileName, false, Encoding.Default);
                csvFile.Write("Sample time in secs;CH1 sample in V;CH2 sample in V\n");

                for (int i = 0; i < waveform_CH1.Capacity; i++)
                {
                    sampleTime = waveform_CH1.StartTime.TotalSeconds
                    + i * waveform_CH1.IntervalPerPoint.TotalSeconds;
                    csvFile.Write(sampleTime.ToString("F6") + columnSeparator
                    + samplesArray_CH1[i].ToString("F3") + columnSeparator
                    + samplesArray_CH2[i].ToString("F3") + rowSeparator);
                }

                Console.WriteLine("Waveform data saved to \"{0}\", file size {1:F3} kB", csvFileName, csvFile.BaseStream.Position / 1024);
                csvFile.Close();
                #endregion
                //----------------------------------------------------------------------------------------------------------------
                #region 3.11 Hardcopy
                //set up the hardcopy
                var Hcpy = driver.HardcopyAndPrinter;
                var Hcpy_DE1 = driver.HardcopyAndPrinter.Destination["DE1"];
                string hcpyFolderPathRTx = @"c:\Temp";
                string hcpyFilePathRTx = hcpyFolderPathRTx + @"\" + "1MA268_screenshot_RTx.png";
                string hcpyFilePathPC = @"c:\Temp\1MA268_screenshot_PC.png";

                Hcpy_DE1.DeviceDestination = HardcopyDevice.File;
                Hcpy_DE1.Color = true;
                Hcpy_DE1.DeviceLanguageOutputFormat = HardcopyDeviceLang.png;
                Hcpy_DE1.WhiteBackground = false;
                Hcpy_DE1.PrintColorSet = HardcopyPrintColorSet.Default4;
                Hcpy_DE1.InverseColors = false;
                Hcpy.FileName = hcpyFilePathRTx;

                //make a screenshot now
                Hcpy.Print();

                //copy the file from the RTx to the PC
                driver.DataManagement.ReadToFileFromInstrument(hcpyFilePathRTx, hcpyFilePathPC);
                #endregion
                //----------------------------------------------------------------------------------------------------------------
                #region 3.12 Reading RTx Folder List
                //read the hcpy_file_path_RTx content
                string folderContent = driver.DataManagement.FileDirectoryContent(hcpyFolderPathRTx);
                //parse the response - the header <UsedMemory>,<FreeMemory>,<ParentFolder>,<ParentFolder>,<FolderContent>
                Match match = new Regex("(\\d+),\\d+,\"[^\"]+\",\"[^\"]+\",(.+)$").Match(folderContent);
                if (match.Success)
                {
                    int usedSize = Int32.Parse(match.Groups[1].Value) / 1024;
                    string info = match.Groups[2].Value;

                    Console.WriteLine("\n\nFolder \"{0}\", size {1} kb, files list:", hcpyFolderPathRTx, usedSize);
                    while ((match = new Regex("\"([^,]+),([^,]+),([^,]+)\"(.+)$").Match(info)).Success)
                    {
                        info = match.Groups[4].Value;
                        Console.WriteLine("File name: \"{0}\", type \"{1}\", size {2} kb",
                            match.Groups[1].Value,
                            match.Groups[2].Value,
                            Int32.Parse(match.Groups[3].Value) / 1024);
                    }
                }
                #endregion
            }
            //----------------------------------------------------------------------------------------------------------------
            #region 3.13 Handling of exceptions
            //3.13.1 Ivi.Driver.IOException handling
            catch (Ivi.Driver.IOException exception)
            {
                exceptionMessage = "\nIO error occured:\n" + exception.Message
                                   + "\n\nStackTrace\n" + exception.StackTrace;
            }
            //----------------------------------------------------------------------------------------------------------------
            //3.13.2 Ivi.Driver.InstrumentStatusException handling
            catch (Ivi.Driver.InstrumentStatusException exception)
            {
                //with this exception the query SYST:ERR? is sent to the instrument until
                //the error query is empty to read out all device errors
                exceptionMessage = "\nInstrument error(s) occured:\n" + exception.Message
                                        + "\n\nStackTrace\n" + exception.StackTrace;
            }
            //----------------------------------------------------------------------------------------------------------------
            //3.13.3 Ivi.Driver.DataArrayTooSmallException handling
            catch (Ivi.Driver.DataArrayTooSmallException exception)
            {
                exceptionMessage = "\nError - waveform array is too small for the data:\n"
                                    + exception.Message
                                    + "\n\nStackTrace\n" + exception.StackTrace;
            }
            //----------------------------------------------------------------------------------------------------------------
            //3.13.4 Ivi.Driver.MaxTimeExceededException handling
            catch (Ivi.Driver.MaxTimeExceededException exception)
            {
                exceptionMessage = "\nTimeout occured:\n"
                                   + exception.Message
                                   + "\n\nStackTrace\n" + exception.StackTrace;
            }

            //----------------------------------------------------------------------------------------------------------------
            //3.13.5 Ivi.Driver.OutOfRangeException handling
            catch (Ivi.Driver.OutOfRangeException exception)
            {
                exceptionMessage = "\nValue out of range exception occured:\n"
                                   + exception.Message
                                   + "\n\nStackTrace\n" + exception.StackTrace;
            }

            //----------------------------------------------------------------------------------------------------------------
            //3.13.6 finally section - always performed regardless whether any exception was raised or not
            finally
            {
                if (exceptionMessage != null) Console.WriteLine(exceptionMessage);
                //wait for a keypress
                Console.WriteLine("\nPress any key to close...");
                Console.ReadKey();
                //close the driver and disposing of the driver object
                driver.Dispose();
            }
            #endregion
            //----------------------------------------------------------------------------------------------------------------
        }
    }
}

//Example for user-defined extensions methods of the RsScope object
namespace RsScopeExtensions
{
    public static class Extensions_of_RsScope
    {
        //----------------------------------------------------------------------------------------------------------------
        #region 3.14 RunSingle() with repeat workaround
        /// <summary> This is an example of extension method added to the RsScope driver
        /// <para>The method repeats the acquisition maximum of 'maxRepeats' times in case of a timeout.</para>
        /// <para>It also gives an option to choose the trigger source or cancel the acquisition process.</para>
        /// </summary>
        public static bool Extended_RunSigleWithRepeat(this RsScope driver, IRsScopeTrigger trigger, int maxRepeats)
        {
            bool repeat;
            bool success = false;
            do
            {
                repeat = false;
                try
                {
                    maxRepeats--;
                    driver.WaveformAcquisition.RunSingle();
                    success = true;
                }

                catch (Ivi.Driver.MaxTimeExceededException)
                {
                    //handle only the exceptions of the MaxTimeExceededException type
                    //all the other types of exceptions will processed by the higher level of catch {}
                    Console.Beep();
                    Console.WriteLine("No trigger event detected within the defined time.");
                    Console.WriteLine("Check your input signal on {0}", trigger.Source);
                    Console.WriteLine("Remaining attempts: {0}", maxRepeats);
                    Console.WriteLine("Press '1' to repeat the acquisition with Trigger on Channel 1.");
                    Console.WriteLine("Press '2' to repeat the acquisition with Trigger on Channel 2.");
                    Console.WriteLine("Press 'ESC' to cancel the acquisition.");
                    
                    System.ConsoleKey choice = Console.ReadKey().Key;
                    if (choice == System.ConsoleKey.D1)
                    {
                        trigger.Source = RohdeSchwarz.RsScope.TriggerSource.Channel1;
                        Console.WriteLine("\n----- repeating the Acquisition #2 with Trigger on Channel 1 ...");
                        repeat = true;
                    }
                    else if (choice == System.ConsoleKey.D2)
                    {
                        trigger.Source = RohdeSchwarz.RsScope.TriggerSource.Channel2;
                        Console.WriteLine("\n----- repeating the Acquisition #2 with Trigger on Channel 2 ...");
                        repeat = true;
                    }
                    Console.WriteLine("\n");
                }
            } while ((repeat == true) && (maxRepeats > 1));

            return success;
            #endregion
            //----------------------------------------------------------------------------------------------------------------
        }
    }
}